#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-dgas-server-config v. 1.1.2
#
# Post-installation script for configuring the gLite DGAS servers
# Guillermo Diez-Andino Sancho <Guillermo.Sancho@cern.ch>
#
# Version info: $Id: glite-dgas-server-config.py,v 1.9.4.21.2.3 2006/03/02 00:00:02 dimeglio Exp $
#
# Usage: python glite-dgas-server-config [-c|-v|-h|--configure|--start|--stop|--status|--help]
#        -c          print configuration
#        -v          print version
#        --configure configure the services
#        --start     start the services
#        --stop      stopt he services
#        --status    check the services status
#        -h,--help   print usage info
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################
#The databases are not removed in the configure function if they already exist

# import system libraries
import os,string
import sys
import getopt
import time

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from mysql import Mysql
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params         			# all config values from the XML file
global glib

class glite_dgas_server:

	def __init__(self):
		self.verbose = 0
		self.version = "1.1.2"
		self.name = "glite-dgas-server"
		self.friendly_name = "gLite DGAS Server"
		self.mysql = Mysql()
		self.hlrProcesses = ["glite-dgas-hlr-had.pl /opt/glite/sbin/glite_dgas_hlrd","glite_dgas_hlrTqd","glite_dgas_hlrServerd"]
		self.paProcesses = ["glite-dgas-hlr-had.pl /opt/glite/sbin/glite_dgas_pad","glite_dgas_paServerd"]
	#-------------------------------------------------------------------------------
	# Banner 
	#-------------------------------------------------------------------------------

	def banner(self):

		print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
		print "          Configuring the %s" % self.friendly_name
		print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
	#-------------------------------------------------------------------------------
	# Copyright 
	#-------------------------------------------------------------------------------

	def copyright(self):

		print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
		print 'See http://eu-egee.org/partners/ for details on the copyright holders'
		print 'For license conditions see the license file or http://eu-egee.org/license.html'

	#-------------------------------------------------------------------------------
	# Version
	#-------------------------------------------------------------------------------

	def showVersion(self):

		print '\n%s-config  v. %s\n' % (self.name,self.version)
    
	#-------------------------------------------------------------------------------
	# Usage
	#-------------------------------------------------------------------------------

	def usage(self,msg = ""):

		if msg:
			print "\n%s" % (msg)
        
		self.copyright()
		self.showVersion()
    
		print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
		os.environ['GLITE_LOCATION'], self.name, self.name)

		print '    -c, --checkconf     print the service configuration'
		print '    -v, --version       print the version of the configuration script'
		print '    -h, --help          print this usage information'
		print '    --configure         configures the dgas services'
		print '    --start			   start the services'
		print '    --stop			   stop the services'
		print '    --status            shows the status of the services'
		print '\n'
	
	def getPID(self,pname=''):
		if pname == '':
			print 'Process name argument is empty'
			return 0
                                                                                                                                                                             
		proclist = os.popen('/bin/ps ax | /bin/grep \"%s\"'% pname)
		proc = proclist.readlines()
		for item in proc:
			if (item.find('/bin/grep') == -1) and (item.find('%s' % pname) != -1) and (item.find('-config.py') == -1):
				rec = string.splitfields(string.strip(item), ' ')
				return int(rec[0])
		else:
			return 0
			
	def stopProcess(self,process,command=""):
		pid = self.getPID(process)
		if pid == 0:
			print "The %s service is not running     " % process
			print "Stopping the %s service           " % process,
			glib.printOkMessage()
		else:
			if command != "":
				os.system(command)
			else:
				os.system("kill -9 %d" % pid)
			time.sleep(2)
			pid = self.getPID(process)
			if pid == 0:
				print "Stopping the %s service           " % process,
				glib.printOkMessage()
				return 0
			else:
				glib.printWarningMessage("Failed to stop the %s service     " % process)
				glib.printFailedMessage()
				return 2
                                                                                                                                                                             
		return 0		
	
	def start(self):
		self.mysql.start()
				
		if params['hlr-server.enabled'] == "true":
			service.startHLR()
		if params['pa-server.enabled'] == "true":
			service.startPA()
		
		#-------------------------------------------------------------------
		# Start Servicetool
		#-------------------------------------------------------------------
		if params['rgma.servicetool.activate'] == 'true':                                                                                          
			errorcode = rgmaServicetool.start()

			if (errorcode == 0):
				print "The gLite R-GMA Service Publisher has been started           ",                                                                                          
				glib.printOkMessage()
			else:
				glib.printErrorMessage("Could not start the gLite R-GMA Service Publisher")
				glib.printErrorMessage("Please verify and re-run the script")
				return 1
		return 0

	def stop(self):
		self.mysql.stop()
		if params['hlr-server.enabled'] == "true":
			service.stopHLR()
		if params['pa-server.enabled'] == "true":
			service.stopPA()
		
		#############################################
		#Stop the servicetool
		#############################################
		if params['rgma.servicetool.activate'] == 'true':
			errorcode = rgmaServicetool.stop()
			if (errorcode):
				print "Stopping R-GMA Service Publisher                             ",
				glib.printFailedMessage()
			else:
				print "Stopping R-GMA Service Publisher                             ",
				glib.printOkMessage()
		return 0
		
	def status(self):
		error_level = 0
		if params['hlr-server.enabled'] == "true":
			if service.statusHLR() != 0:
				error_level = 1
		if params['pa-server.enabled'] == "true":
			if service.statusPA() != 0:
				error_level = 1
		return error_level
	
	#############################################
	# All the configuration code goes here
	#############################################
	def configure(self):
	
		#############################################
		#Stopping the services
		#############################################
		print "Stopping the services " 
		if self.stop() != 0:
			return 1
		
		#############################################
		#Checking the host certificates
		#############################################
		if glib.check_certs(params) != 0:
			print "Problems checking the host certificates %s" % service.friendly_name
			sys.exit(1)
			
		#############################################
		#Checking the grid-mapfile
		#############################################
		if not os.path.exists(params['host.gridmapfile']):
			print("Creating an empty grid-mapfile under %s" %params['host.gridmapfile'])
			os.system("/bin/touch %s" % params['host.gridmapfile'])
		else:
			print("The grid-mapfile exists under %s" %params['host.gridmapfile']),
			glib.printOkMessage()
		
		#############################################
		#Configuring the security utililities
		#############################################
		if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
			print "\nInstalling gLite Security Utilities               ",
			glib.printFailedMessage()
			return 1
		print "\nInstalling gLite Security Utilities                   ",
		glib.printOkMessage()
		
		#############################################
		#Starting mysql
		#############################################
		self.mysql.start()
		
		#############################################
		#Configuring root password for mysql
		#############################################
		if not params.has_key('mysql.root.password'):
			params['mysql.root.password'] = ""
		if not params['mysql.root.password'] == "":
			self.mysql.setpassword(params['mysql.root.password'], os.environ['HOSTNAME'])

		############################################
		# Configuring the PA service
		############################################
		if params['pa-server.enabled'] == "true":
			service.configurePA()

		############################################
		# Configuring the HLR services
		############################################
		if params['hlr-server.enabled'] == "true":
			service.configureHLR()
			
		############################################
		# Configuring the rgma servicetool
		############################################
		if self.configureServiceTool() == 1:
			return 1
                                                                                                                                                               
		print "Configuring the R-GMA Service Publisher                      ",
		glib.printOkMessage()
			
		############################################
		# Stopping mysql
		############################################	
		self.mysql.stop	()
		
		return 0
	
	def configureServiceTool(self):
		serviceTypes = ["org.glite.dgas.server.pa","dgas.server.hlr"]
		serviceScripts = ["${GLITE_LOCATION}/sbin/glite_dgas_pad status","${GLITE_LOCATION}/sbin/glite_dgas_hlrd status"]
		
		if params['rgma.servicetool.activate'] == "true":
			for idx in xrange(len(serviceTypes)):
				serviceId = "%s_%s" % (glib.fq_hostname,serviceTypes[idx])
				servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
				glib.printInfoMessage("Configuring servicetool instance %s" %servicetoolInstance)
                                                                                                                                                               
				# set params
				servicetoolInstance.setServiceName(serviceId)
				servicetoolInstance.setServiceType(serviceTypes[idx])
				servicetoolInstance.setServiceVersion(self.version)
				servicetoolInstance.setStatusScript(serviceScripts[idx])
				servicetoolInstance.setEndpoint('not available')

				# add instance to the gLite configuration
				if servicetoolInstance.add():
					return 1
			
			if rgmaServicetool.configure(glib):
				# error in configuring servicetool
				return 1
		return 0			

	def startPA(self):
		paServer =  '%s/sbin/glite_dgas_pad start' % (os.environ['GLITE_LOCATION'])
		returnCode = 0
		
		self.stopPA()
		os.system(paServer)
		
		for process in self.paProcesses:
			if self.getPID(process) == 0:
				glib.printErrorMessage("The %s daemon has not been properly started" %process)
				returnCode = 1
				
		return returnCode
		
	def stopPA(self):
		returnCode = 0
		for process in self.paProcesses:
			if self.getPID(process) != 0:
				self.stopProcess(process)
				if self.getPID(process) != 0:
					glib.printErrorMessage("The process %s has not been properly stopped" %process)
					returnCode = 1
			else:
				print("The process %s is already stopped" %process),
				glib.printOkMessage()
		return returnCode
		
	def statusPA(self):
		error_level = 0
		for process in self.paProcesses:
			pid = self.getPID(process)
			if pid != 0:
				print "The process %s is running" %process
			else:
				print "The process %s is not running" %process
				error_level = 1
		return error_level
		
	#############################################
	# Configuing the PA service
	#############################################
	def configurePA(self):
		print "Configuring the PA server"
		path = "%s/etc/dgas_pa.conf" % os.environ['GLITE_LOCATION']
		pathBak = "%s/etc/dgas_pa.conf.bak" % os.environ['GLITE_LOCATION']
	
		if os.path.exists(pathBak):
			os.remove(pathBak)
		if os.path.exists(path):
			os.rename(path,pathBak)

		#Check if hlr-server.enabled = true
		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		# DataBase related
		file.write("pa_sql_server = \"%s\"\n" % params['pa-server.db.server'])
		file.write("pa_sql_user = \"%s\"\n" % params['pa-server.db.user'])
		file.write("pa_sql_password = \"%s\"\n" % params['pa-server.db.password'])
		file.write("pa_sql_dbname = \"%s\"\n" % params['pa-server.db.name'])
		file.write("pa_def_port = \"%s\"\n" % params['pa-server.port'])
		#log and lock files
		file.write("pa_def_log = \"%s\"\n" % params['pa-server.logfile'])
		file.write("pa_def_lock = \"%s\"\n" % params['pa-server.lockfile'])
		file.write("pa_had_def_lock = \"%s\"\n" % params['pa-server.had.lockfile'])
		file.write("pa_server_contact = \"%s\"\n" %params['pa-server.contact'])
		file.write("pricing_scheme = \"%s\"\n" % params['pa.pricing.schema'])
		file.write("pa_price_dll_name = \"%s\"\n" % params['pa.pricing.dll.name'])
		file.write("def_price_minTTL = \"%s\"\n" % params['pa.pricing.ttl'])
		file.close()
		print("%s configuration file created " % path),
		glib.printOkMessage()
	
		#DATABASE creation
		if self.mysql.existsDB(params['pa-server.db.name'],params['mysql.root.password']) != 0:
			os.system("%s/sbin/glite-dgas-pad-dbcreate %s" %(os.environ['GLITE_LOCATION'],params['mysql.root.password']))
			if self.mysql.existsDB(params['pa-server.db.name'],params['mysql.root.password']) != 0:
				glib.printErrorMessage("Problems to create the PA database with name %s" %params['pa-server.db.name'])
				return 1
			else:
				print("PA database with name: %s successfully created" %params['pa-server.db.name']),
				glib.printOkMessage()
		else:
			print("The PA database with name: %s already exists" %params['pa-server.db.name']),
			glib.printOkMessage()
			
		return 0			
	
	def startHLR(self):
		returnCode = 0
		hlrServer = '%s/sbin/glite_dgas_hlrd start' % (os.environ['GLITE_LOCATION'])
		
		self.stopHLR()
		os.system(hlrServer)
		for process in self.hlrProcesses:
			if self.getPID(process) == 0:
				glib.printErrorMessage("The %s daemon has not been properly started" %process)
				returnCode = 1

		return returnCode
	
	def stopHLR(self):
		returnCode = 0

		for process in self.hlrProcesses:
			if self.getPID(process) != 0:
				self.stopProcess(process)
				if self.getPID(process) != 0:
					glib.printErrorMessage("The process %s has not been properly stopped" %process)
					returnCode = 1
			else:
				print("The process %s is already stopped" %process),		
				glib.printOkMessage()
		return returnCode
		
	def statusHLR(self):
		error_level = 0
		for process in self.hlrProcesses:
			pid = self.getPID(process)
			if pid != 0:
				print "The process %s is running" %process
			else:
				print "The process %s is not running" %process
				error_level = 1
		return error_level
	
	#############################################
	# Configuing the HLR services
	#############################################
	def configureHLR(self):
		returnCode = 0

		print "Configuring the HLR server"
		
		#Adding the host certificate to the grid-mapfile
		subject = glib.get_certificate_subject(params['host.certificate.file'],"/")
		print "Looking for the host certificate %s in %s" %(subject,params['host.gridmapfile'])
	
		found = 0
		file = open(params['host.gridmapfile'],'r')
		for line in file.xreadlines():
			if not line.strip().startswith('#') and line.find("\"%s\"" %subject) != -1:
				found = 1
				break
		file.close()
		
		if found == 0:
			file = open(params['host.gridmapfile'],'a')
			file.write("\"%s\"" %subject)
			print("Added host certificate %s to %s" %(subject,params['host.gridmapfile'])),
			glib.printOkMessage()
			file.close()
		else:
			print("Host certificate with subject %s already exists in %s" %(subject,params['host.gridmapfile'])),
			glib.printOkMessage()
		
		print "Creating the %s user and group %s" %(params['hlr-server.user'],params['hlr-server.user'])
		gid = glib.add_group(params['hlr-server.user'])
		glib.printInfoMessage("Group %s has been added or verified" % params['hlr-server.user'])
		glib.add_user(params['hlr-server.user'],params['hlr-server.user'])
		
		#Creating configuration files
		path = "%s/etc/dgas_hlr.conf" % os.environ['GLITE_LOCATION']
		pathBak = "%s/etc/dgas_hlr.conf.bak" % os.environ['GLITE_LOCATION']
		print "Creating the %s configuration file " %path

		if os.path.exists(pathBak):
			os.remove(pathBak)
		if os.path.exists(path):
			os.rename(path,pathBak)
	
		file = open(path, 'w')
		# DataBase related
		file.write("hlr_sql_server = \"%s\"\n" % params['hlr-server.db.server'])
		file.write("hlr_sql_user = \"%s\"\n" % params['hlr-server.db.user'])
		file.write("hlr_sql_password = \"%s\"\n" % params['hlr-server.db.password'])
		file.write("hlr_sql_dbname = \"%s\"\n" % params['hlr-server.db.name'])
		file.write("hlr_tmp_sql_dbname = \"%s\"\n" % params['hlr-server.db.tmp.name'])
		# GridMap file 
		file.write("gridmapFile = \"%s\"\n" % params['host.gridmapfile'])
		#hostProxyFile
		file.write("hostProxyFile = \"%s\"\n" % params['hlr-server.proxyfile'])
		# hlr user
		file.write("hlr_user = \"%s\"\n" % params['hlr-server.user'])
		#hlr port
		file.write("hlr_def_port = \"%s\"\n" % params['hlr-server.port'])
		#dgas var dir
		file.write("dgas_var_dir = \"%s\"\n" % params['hlr-server.dgas.var'])
		#log and lock files
		file.write("hlr_def_log = \"%s\"\n" % params['hlr-server.logfile'])
		file.write("hlr_qmgr_def_log = \"%s\"\n" % params['hlr-transaction-manager.logfile'])
		file.write("hlr_def_lock = \"%s\"\n" % params['hlr-server.lockfile'])
		file.write("hlr_qmgr_def_lock = \"%s\"\n" % params['hlr-transaction-manager.lockfile'])
		file.write("hlr_qmgr_expPeriod = \"%s\"\n" % params['hlr-transaction-manager.expiration.period'])
		file.write("hlr_qmgr_qDepth = \"%s\"\n" % params['hlr-transaction-manager.queue.depth'])
		file.write("hlr_qmgr_tPerIter = \"%s\"\n" % params['hlr-transaction-manager.processed'])
		file.write("hlr_qmgr_pollPeriod = \"%s\"\n" % params['hlr-transaction-manager.interval'])
		file.write("thread_number = \"%s\"\n" % params['hlr-server.thread.number'])
		#had lock file
		file.write("hlr_had_def_lock = \"%s\"\n" % params['hlr-server.had.lockfile'])
		file.close()
		print "%s configuration file created" % path,
		glib.printOkMessage()
		
		#Database and user creation
		if ((self.mysql.existsDB(params['hlr-server.db.name'],params['mysql.root.password']) != 0) \
			or (self.mysql.existsDB(params['hlr-server.db.tmp.name'],params['mysql.root.password']) != 0)):
			os.system("%s/sbin/glite-dgas-hlrd-dbcreate %s" %(os.environ['GLITE_LOCATION'],params['mysql.root.password']))
			if self.mysql.existsDB(params['hlr-server.db.name'],params['mysql.root.password']) != 0:
				glib.printErrorMessage("Problems to create the %s database" %params['hlr-server.db.name'])
				returnCode = 1
			else:
				print("Database %s successfully created" %params['hlr-server.db.name']),
				glib.printOkMessage()	
					
			if self.mysql.existsDB(params['hlr-server.db.tmp.name'],params['mysql.root.password']) != 0:
				glib.printErrorMessage("Problems to create the %s database" %params['hlr-server.db.tmp.name'])
				returnCode = 1
			else:
				print("Database %s successfully created" %params['hlr-server.db.tmp.name']),
				glib.printOkMessage()
		else:
			print("The hlr databases %s and %s already existed" %(params['hlr-server.db.name'],params['hlr-server.db.tmp.name'])),
			glib.printOkMessage()
		
		return returnCode	
	
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------
def set_env():
	
	glib.export('GLITE_LOCATION');
	glib.export('GLITE_LOCATION_VAR');
	if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
		os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
	glib.export('GLITE_LOCATION_LOG');
	if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
		os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
	glib.export('GLITE_LOCATION_TMP');
	if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
		os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

	glib.export('GLOBUS_LOCATION')
	
	# bin and lib paths
	glib.addEnvPath("PATH","%s/bin:%s/sbin,%s/libexec" % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))
	glib.addEnvPath("LD_LIBRARY_PATH","%s/lib:%s/lib:/opt/gpt/lib" % (os.environ['GLOBUS_LOCATION'],os.environ['GLITE_LOCATION']))
	#glib.addEnvPath("MYSQL_UNIX_PORT","/var/lib/mysql/mysql.sock")
	
	# Set environment
	glib.setUserEnv()	
	

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':
	# The script must be run as root
	if not os.geteuid()==0:
		glib.printFailedMessage("This script must be run as root")
		sys.exit(1)
	
	# Get an instance of the library class
	glib = gLib()
	
	# Load parameters
	params = ConfigParams() 
	glib.loadConfiguration(["%s/../glite-dgas-server.cfg.xml" % glib.getScriptPath(), "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()], params)
	glib.loadConfiguration("%s/../glite-security-utils.cfg.xml" % glib.getScriptPath(), params) 	
	
	try:
		opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
		for o, a in opts:
			if o == "--siteconfig":
				params['site.config.url'] = a
				break
	except getopt.GetoptError:
		pass
    
	# set verbosity
	if params.has_key('glite.installer.verbose'):
		if (params['glite.installer.verbose'] == "true"):
			verbose=1
		else:
			verbose=0
	else:
		verbose = 0
	glib.verbose = verbose
	
	# Set up the environment
	set_env()
	service = glite_dgas_server()
	
	# Instantiate the rgma servicetool class
	if not params.has_key('rgma.servicetool.activate'):
		params['rgma.servicetool.activate'] = 'true'
	if params['rgma.servicetool.activate'] == 'true':
		rgmaServicetool = gliteRgmaServicetool()
		rgmaServicetool.verbose = verbose
                                                                                                  
	path = '%s/etc' % os.environ['GLITE_LOCATION']
	if not os.path.exists(path):
		glib.printErrorMessage("%s/etc does not exist! Please verify your gLite installation") % os.environ['GLITE_LOCATION']
		sys.exit (1)
	else:
		print '%s/etc exists' % os.environ['GLITE_LOCATION'],
		glib.printOkMessage()

	#Checking that at least the hlr or the pa is activated
	if params['hlr-server.enabled'] != "true" and params['pa-server.enabled'] != "true":
		glib.printErrorMessage("At least one of either the hlr-server or the pa-server should be enabled in the configuration file ")
		sys.exit(1)
	
	# Command line opts if any
	try:
		opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','stop','start','status','configure','siteconfig='])
	except getopt.GetoptError:
		service.usage(msg = "[FAILED] Unknown options(s)")
		sys.exit(1)

	# Check cli options
	for o, a in opts:
		if o in ("-h", "--help"):
			service.usage()
			sys.exit(0)
		if o in ("-v", "--version"):
			service.showVersion()
			sys.exit(0)
		if o in ("-c", "--checkconf"):
			service.copyright()
			service.showVersion()
			glib.print_params(params)
			sys.exit(0)
   		if o == "--start":
			if service.start() == 0:
				print "The %s was successfully started" % service.friendly_name,
				glib.printOkMessage()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while starting the %s service" % service.friendly_name)
				sys.exit(1)
		if o == "--stop":
			if service.stop() == 0:
				print("The service %s was successfully stopped" % service.friendly_name),
				glib.printOkMessage()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while stopping the %s service" % service.friendly_name)
				sys.exit(1)
		if o == "--status":
			sys.exit(service.status())
		if o == "--configure":
			if service.configure() == 0:
				print("%s configuration successfully completed " % service.friendly_name),
				glib.printOkMessage()
				glib.registerService()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while configuring the %s service" % service.friendly_name)
				sys.exit(1)
 
	# Print configuration parameters
	if verbose:
		glib.print_params(params)
		service.copyright()
		service.showVersion()
		service.banner()
	service.usage()	
